unit ReadOnlyAddIn;

// Mung the read-only attribute of a file.
// Delphi's source editor has a ReadOnly menu item in its context menu.
// The state of the menu item reflects the buffer's read-only flag.
// When Delphi opens a readonly file, it sets the readonly flag, but the
// user can change the readonly flag manually. Changing the flag does not
// affect the file attribute.
//
// This IDE add-in ties the menu item to the file attributes. When the user
// choose the Read Only menu item, the add-in grabs the event and sets
// the file's attribute to match the buffer's flag.
//
// Copyright  2002 Tempest Software, Inc.
//
// You can use this file for personal use or in a commercial setting.
// You can even include it on a collection of Delphi tools. Just don't remove
// the copyright notice or pretend that you wrote it.

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs,
  Menus, ActnList, ToolsAPI;

type
  TReadOnlyMunger = class(TNotifierObject, IOTAWizard)
  private
    { Private declarations }
    SaveExecute: TNotifyEvent;
    SaveAction: TBasicAction;
    procedure ReadOnlyExecute(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterPackageWizard(TReadOnlyMunger.Create);
end;

constructor TReadOnlyMunger.Create;
var
  Form: TForm;
  Menu: TPopupMenu;
  Action: TBasicAction;
  I, J: Integer;
begin
  // Find the editor's popup menu and then the Read-Only action item.
  for I := 0 to Screen.FormCount-1 do
  begin
    Form := Screen.Forms[I];
    Menu := Form.FindComponent('EditorLocalMenu') as TPopupMenu;
    if Menu <> nil then
      for J := 0 to Menu.Items.Count-1 do
      begin
        Action := Menu.Items[J].Action;
        if Assigned(Action) and SameText(Action.Name, 'ecReadOnlyItem') then
        begin
          SaveExecute := Action.OnExecute;
          SaveAction := Action;
          Action.OnExecute := ReadOnlyExecute;
          Exit;
        end;
      end;
  end;
  raise Exception.Create('Cannot find ecReadOnlyItem');
end;

destructor TReadOnlyMunger.Destroy;
begin
  SaveAction.OnExecute := SaveExecute;
  inherited;
end;

procedure TReadOnlyMunger.Execute;
begin
// Stub: Delphi never calls Execute for this add-in.
end;

function TReadOnlyMunger.GetIDString: string;
begin
  Result := 'Tempest Software.ReadOnly Munger';
end;

function TReadOnlyMunger.GetName: string;
begin
  Result := 'Read-Only Munger';
end;

function TReadOnlyMunger.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

// The user toggled the read-only flag of the current buffer.
// If the buffer has a corresponding file, toggle the flag
// of that file, too.
procedure TReadOnlyMunger.ReadOnlyExecute(Sender: TObject);
var
  Buffer: IOTAEditBuffer;
  Attr: DWORD;
begin
  SaveExecute(Sender);
  Buffer := (BorlandIDEServices as IOTAEditorServices).TopBuffer;
  if Buffer = nil then
    Exit;

  // If the file has a virtual file system, skip it. A VFS has the ability
  // to query the read-only state, but not change it.
  if Buffer.Module.FileSystem <> '' then
    Exit;

  Attr := GetFileAttributes(PChar(Buffer.FileName));
  if Attr = $FFFFFFFF then
    // Cannot tell whether the file is read-only.
    Exit;

  if Buffer.IsReadOnly then
    Attr := Attr or File_Attribute_ReadOnly
  else
    Attr := Attr and not File_Attribute_ReadOnly;
  SetFileAttributes(PChar(Buffer.FileName), Attr);
end;

end.
 